package io.tswf.minecraft.blockventure.core.transmutation.transmute

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.minecraft.blockventure.core.api.registry.BiomeRegistry
import io.tswf.minecraft.blockventure.core.api.registry.Register
import io.tswf.minecraft.blockventure.core.transmutation.view.ViewDynamicRegistryManagerImmutable
import io.tswf.transmutations.common.annotation.ClassReference
import io.tswf.transmutations.common.annotation.MethodDescription
import io.tswf.transmutations.common.constant.MethodName
import io.tswf.transmutations.transmute.annotation.AtExit
import io.tswf.transmutations.transmute.annotation.Self
import io.tswf.transmutations.transmute.annotation.Transmute
import io.tswf.transmutations.transmute.meta.MissingTargetStrategy
import net.minecraft.registry.DynamicRegistryManager
import net.minecraft.registry.Registry
import net.minecraft.registry.RegistryKey
import net.minecraft.registry.RegistryKeys
import net.minecraft.registry.SimpleRegistry

import java.util.stream.Stream

@Slf4j
@CompileStatic
class TransmutationDynamicRegistryManagerImmutable {

    static void injectIntoDynamicRegistries(Map registriesMap) {
        log.info("Injecting into ${registriesMap}!")
        def biomeRegistry = (Registry) registriesMap[RegistryKeys.BIOME]
        if (biomeRegistry != null) {
            for (def biomeToRegisterEntry : BiomeRegistry.registeredBiomes) {
                log.info("Registering ${biomeToRegisterEntry.value} biome by ${biomeToRegisterEntry.key} id!")
                Register.register(biomeRegistry, biomeToRegisterEntry.value, biomeToRegisterEntry.key)
            }
        }
    }

    @Transmute(
            targetClass = @ClassReference(DynamicRegistryManager.ImmutableImpl),
            targetMethod = @MethodDescription(
                    name = MethodName.CONSTRUCTOR,
                    returnType = @ClassReference(ClassReference.VOID),
                    parameterTypeClasses = [ List ]
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void injectIntoDynamicRegistries(List registriesMap, @Self Object self) {
        def view = (ViewDynamicRegistryManagerImmutable) self
        def mutableRegistries = new HashMap<>(view.registries)
        view.registries = mutableRegistries

        injectIntoDynamicRegistries(mutableRegistries)
    }

    @Transmute(
            targetClass = @ClassReference(DynamicRegistryManager.ImmutableImpl),
            targetMethod = @MethodDescription(
                    name = MethodName.CONSTRUCTOR,
                    returnType = @ClassReference(ClassReference.VOID),
                    parameterTypeClasses = [ Map ]
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void injectIntoDynamicRegistries(Map registriesMap, @Self Object self) {
        def view = (ViewDynamicRegistryManagerImmutable) self
        def mutableRegistries = new HashMap<>(view.registries)
        view.registries = mutableRegistries

        injectIntoDynamicRegistries(mutableRegistries)
    }

    @Transmute(
            targetClass = @ClassReference(DynamicRegistryManager.ImmutableImpl),
            targetMethod = @MethodDescription(
                    name = MethodName.CONSTRUCTOR,
                    returnType = @ClassReference(ClassReference.VOID),
                    parameterTypeClasses = [ Stream ]
            ),
            onMissing = MissingTargetStrategy.THROW
    )
    @AtExit
    static void injectIntoDynamicRegistries(Stream registriesMap, @Self Object self) {
        def view = (ViewDynamicRegistryManagerImmutable) self
        def mutableRegistries = new HashMap<>(view.registries)
        view.registries = mutableRegistries

        injectIntoDynamicRegistries(mutableRegistries)
    }

}
